
#include "Timer0.h"
#include "ISR_Defs.h"
#include "Konstanten.h"

#define PORT_AUSG	PORTA
#define DDR_AUSG	DDRA
#define PIN_EING	PINC
#define DDR_EING	DDRC
#define PULLUP_EING	PORTC


// -> fr Tastenentprellung 2ms*8 * 4

uint8_t Timer0Start;
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#define ANZ_TASTENZEILEN	8
volatile uint8_t TasteStatus[ANZ_TASTENZEILEN];
volatile uint8_t TasteFertig[ANZ_TASTENZEILEN];
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void TastenfeldInit()
{
 DDR_AUSG = 0xFF;
 PORT_AUSG = 0xFF;
 DDR_EING = 0x00;
 PULLUP_EING = 0xFF;
};
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void Timer0_Init()
{
 TastenfeldInit();
 TCCR0  = (0 << CS02) | (1 << CS01) | (1 << CS00);	// clk/64
 Timer0Start = 0xFF - (K_CPUTakt / 64/ 1000 * 2 - 1);
 TCNT0 = Timer0Start;
 TIMSK |= (1 << TOIE0);
};
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
ISR(TIMER0_OVF_vect)
{
 TCNT0 = Timer0Start;
 IRStatus |= 1 << B_TIMER0_OVF;

// Tastenentprellung (frei) nach Peter Dannegger
// ---------------------------------------------
 static uint8_t cnt0[ANZ_TASTENZEILEN],cnt1[ANZ_TASTENZEILEN];
 uint8_t i;

 static uint8_t verz;
 verz = (verz + 1) & 0x07;
 if (!verz)
 {
  for (uint8_t z = 0; z < ANZ_TASTENZEILEN; z++)	// zeilenweise Ausgnge schalten
  {
   PORT_AUSG = ~(1 << z); 
   i = TasteStatus[z] ^ ~PIN_EING;
   cnt0[z] = ~(cnt0[z] & i);				// reset or count ct0
   cnt1[z] = cnt0[z] ^ (cnt1[z] & i);	// reset or count ct1
   i &= cnt0[z] & cnt1[z];				// count until roll over ?
   TasteStatus[z] ^= i;					// then toggle debounced state
   TasteFertig[z] |= TasteStatus[z] & i; // 0->1: key press detect
  }
  PORT_AUSG = 0xFF; 
 }
// ---------------------------------------------
};
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
uint8_t TasteGedrueckt1(uint8_t zei,uint8_t spa)
{
 cli();
 uint8_t Maske = TasteFertig[zei] & spa;
 TasteFertig[zei] ^= Maske;
 sei();
 return Maske;
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
uint8_t TasteGedrueckt2(uint8_t zei1,uint8_t spa1, uint8_t zei2,uint8_t spa2)
{
 cli();
 volatile uint8_t Maske1 = TasteFertig[zei1] & TasteStatus[zei1] & spa1;
 volatile uint8_t Maske2 = TasteFertig[zei2] & TasteStatus[zei2] & spa2;
 if (Maske1 && Maske2)
 {
  TasteFertig[zei1] ^= Maske1;
  TasteFertig[zei2] ^= Maske2;
  Maske1 = 0xFF;
 }
 else
 {
  TasteFertig[zei1] &= (TasteStatus[zei1] & spa1) | ~spa1;
  TasteFertig[zei2] &= (TasteStatus[zei2] & spa2) | ~spa2;
  Maske1 = 0x00;
 }
 sei();
 return Maske1;
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
