
// -------------------------------------------------------
// USART Atmega162
// -------------------------------------------------------



#include "USART.h"
#include "ISR_Defs.h"





volatile static uint8_t USARTerror=0;
static uint8_t volatile* USARTpuffer;

volatile static uint8_t* USARTpos;	// Zeiger auf aktuelle Position
volatile static uint8_t DataLen;		// aktuelle Lnge der Nachricht

static uint8_t AdressePlatine;
// --------------------------------------------------------
void RS485in();
void RS485out();
uint8_t CheckSum(volatile uint8_t* buf);
// --------------------------------------------------------
void USART_Init(uint8_t* buf,uint8_t adr)
{
 USARTpuffer=buf;
 AdressePlatine=adr;
 V_DDR_TR|=1 << B_PIN_TR;
 V_DDR_RS_IN_OUT|=1 << B_PIN_RS_IN_OUT;
// Baudrate einstellen
 V_UBRRH = HIGH(W_UBRR);
 V_UBRRL = LOW(W_UBRR);
// Frame-Format 9 Bit, 2 Stopp-Bit
 V_UCSRC = W_UCSRC;
// Empfang + Senden
 V_UCSRB = W_UCSRB;
 RS485in();
};
// --------------------------------------------------------
ISR(V_USART_RXC_vect)
{
 uint8_t AdrBit;
 uint8_t DataByte;
 IRStatus |= 1 << B_USART_BUSY;
// Fehler speichern
 USARTerror |= V_UCSRA & W_USARTerror_Maske;
// Adresse? Daten?
 AdrBit=V_UCSRB & (1 << B_RXB8);
 DataByte=V_UDR;
 if (AdrBit)
 {
// /* TEST */ PORTD ^= (1 << 6);	// -->> fehlerfrei
// Adresse lesen
  if (DataByte==AdressePlatine)
  {
   V_UCSRA = (0 << B_MPCM);
   USARTpos=USARTpuffer;
   DataLen=0;
  }
 }
// Daten eintragen
 *USARTpos++=DataByte;
 if (USARTpos-USARTpuffer == 1+1) 
 {
  DataLen=DataByte;
 }
 /* TEST */ if ((USARTpos-USARTpuffer) == (DataLen-1)) PORTD ^= (1 << 6);	// geht bis DataLen-1
// fertig?
 if (USARTpos-USARTpuffer == DataLen) 
 {
// /* TEST */ PORTD ^= (1 << 6);	// -->> wird nur 1x erreicht
  V_UCSRA = 1 << B_MPCM;
  IRStatus |= 1 << B_USART_RXC;
 }
};
// --------------------------------------------------------
ISR(V_USART_UDRE_vect)
{
// Start mit Adress-Bit: gesetzt in 'DatenUSARTout()'

 V_UDR=*USARTpos++;
 V_UCSRB&=~(1 << B_TXB8);

 if (USARTpos==USARTpuffer+DataLen)
 {
  V_UCSRB &= ~(1 <<  B_UDRIE);
  V_UCSRB |= 1 << B_TXCIE;
 }
};
// --------------------------------------------------------
ISR(V_USART_TXC_vect)
{
 V_UCSRB &= ~(1 << B_TXCIE);
 IRStatus &= ~(1 << B_USART_BUSY);
 IRStatus|=1 << B_USART_TXC;
};
// --------------------------------------------------------
bool DatenUSARTin()
{
 IRStatus &= ~(1 << B_USART_BUSY);
 if (USARTerror)
  return false;
 if (CheckSum(USARTpuffer) ^ USARTpuffer[DataLen-1])
  return false;
 return true;
};
// --------------------------------------------------------
void DatenUSARTout()
{
 IRStatus|=1 << B_USART_BUSY;
 USARTpos=USARTpuffer;
 DataLen=USARTpuffer[1];
 USARTpuffer[DataLen-1]=CheckSum(USARTpuffer);
 RS485out();
};
// --------------------------------------------------------
void RS485in()
{
 V_UCSRA = 1 << B_MPCM;
 V_UCSRB |= 1 << B_RXCIE;	
 V_PORT_RS_IN_OUT &= ~(1 << B_PIN_RS_IN_OUT);
};
// --------------------------------------------------------
void RS485out()
{
 V_PORT_RS_IN_OUT |= (1 << B_PIN_RS_IN_OUT);
 V_UCSRB |= 1 << B_TXB8;
 V_UCSRB |= 1 << B_UDRIE;	
};
// --------------------------------------------------------
uint8_t CheckSum(volatile uint8_t* buf)
{
 uint8_t chk=0x1A;
 for (uint8_t i=0; i < buf[1]-1; i++)
 {
  chk ^= buf[i];
 }
 return chk;
};
// --------------------------------------------------------




