
#ifndef _USART_DEFS_H_
#define _USART_DEFS_H_

#include <stdlib.h>
#include <stddef.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include "Konstanten.h"

// USART
//#define K_BAUD		76800UL
//#define K_BAUD		19200UL
//#define K_BAUD		4800UL
#define K_BAUD		300UL
	// Berechnungen BAUD-Rate
#define W_UBRR ((K_CPUTakt+K_BAUD*8)/(K_BAUD*16)-1)			// clever runden
#define BAUD_REAL (K_CPUTakt/(16*(W_UBRR+1)))				// Reale Baudrate
#define BAUD_ERROR ((BAUD_REAL*1000)/K_BAUD)				// Fehler in Promille, 1000 = kein Fehler.
 
#if ((BAUD_ERROR < 990) || (BAUD_ERROR > 1010))
  #error Systematischer Fehler der Baudrate (USART) grsser 1% und damit zu hoch! 
#endif 

	// Berechnungen Umschaltverzgerung IN/OUT (vorerst Lnge von 1 Byte)
#define BITS		12
#define DELAYus		(1000000*BITS/K_BAUD)
#define DELAYticks	(DELAYus*650/(1000000000/K_CPUTakt))

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifdef _AVR_IOM162_H_
 #define V_UBRRH	UBRR1H
 #define V_UBRRL	UBRR1L
 
 #define V_UCSRC	UCSR1C
 #define V_UCSRB	UCSR1B
 #define V_UCSRA	UCSR1A
 #define V_UDR		UDR1

 #define B_RXB8		RXB81
 #define B_TXB8		TXB81
 #define B_MPCM		MPCM1
 #define B_UDRIE	UDRIE1
 #define B_TXCIE	TXCIE1
 #define B_RXCIE	RXCIE1


// Frame-Format 9 Bit, 2 Stopp-Bit
 #define W_UCSRC	((1 << URSEL1) | (0 << UMSEL1) | (0 << UPM11) | (0 << UPM10) | (1 << USBS1) | (1 << UCSZ11) | (1 << UCSZ10) | (0 << UCPOL1))
// Empfang + Senden (RXCIE, UDRIE, TXCIE bei Bedarf setzen)
 #define W_UCSRB	((0 << RXCIE1) | (0 << TXCIE1) | (0 << UDRIE1) | (1 << RXEN1) | (1 << TXEN1) | (1 << UCSZ12))
// MPCM setzen, fr (gelegentlichen) Empfang 
 #define W_UCSRA	(1 << MPCM1)

 #define V_USART_RXC_vect	USART1_RXC_vect
 #define V_USART_UDRE_vect	USART1_UDRE_vect
 #define V_USART_TXC_vect	USART1_TXC_vect

 #define B_PIN_RS_IN_OUT	PB1
 #define V_PORT_RS_IN_OUT	PORTB
 #define V_DDR_RS_IN_OUT	DDRB

 #define B_PIN_TR			PB3
 #define V_DDR_TR			DDRB

 #define W_USARTerror_Maske	((1 << FE1) | (1 << DOR1) | (1 << UPE1))
#endif	//  _AVR_IOM162_H_
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#ifdef _AVR_IOTN2313_H_
 #define V_UBRRH	UBRRH
 #define V_UBRRL	UBRRL
 
 #define V_UCSRC	UCSRC
 #define V_UCSRB	UCSRB
 #define V_UCSRA	UCSRA
 #define V_UDR		UDR

 #define B_RXB8		RXB8
 #define B_TXB8		TXB8
 #define B_MPCM		MPCM
 #define B_UDRIE	UDRIE
 #define B_TXCIE	TXCIE
 #define B_RXCIE	RXCIE

// Frame-Format 9 Bit, 2 Stopp-Bit
 #define W_UCSRC	((0 << UMSEL) | (0 << UPM1) | (0 << UPM0) | (1 << USBS) | (1 << UCSZ1) | (1 << UCSZ0) | (0 << UCPOL))
// Empfang + Senden (RXCIE, UDRIE, TXCIE bei Bedarf setzen)
 #define W_UCSRB	((0 << RXCIE) | (0 << TXCIE) | (0 << UDRIE) | (1 << RXEN) | (1 << TXEN) | (1 << UCSZ2))
// MPCM setzen, fr (gelegentlichen) Empfang 
 #define W_UCSRA	(1 << MPCM)

 #define V_USART_RXC_vect	USART_RX_vect
 #define V_USART_UDRE_vect	USART_UDRE_vect
 #define V_USART_TXC_vect	USART_TX_vect

 #define B_PIN_RS_IN_OUT	PD2
 #define V_PORT_RS_IN_OUT	PORTD
 #define V_DDR_RS_IN_OUT	DDRD

 #define B_PIN_TR			PD1
 #define V_DDR_TR			DDRD

 #define W_USARTerror_Maske	((1 << FE) | (1 << DOR) | (1 << UPE))
#endif	// _AVR_IOTN2313_H_
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~




void USART_Init(uint8_t* buf,uint8_t adr);
bool DatenUSARTin();
void DatenUSARTout();


ISR(V_USART_RXC_vect);
ISR(V_USART_UDRE_vect);
ISR(V_USART_TXC_vect);


















#endif	// _USART_DEFS_H_
